#include "Image.h"

Image::Image()
{
   alpha = 255;
	scaleX = 0;
	scaleY = 0;
	angle = 0;
	//	By default we don't want to process rotation since these are slower
	useRotation = false;
	useHardware = false; //	and HW doesn't support alpha (need colour masking for transparencies)
}

Image::~Image()
{
	clear();
}

bool Image::loadImage(string name)
{
   uint currentI = (uint)images.size();

	images.push_back( IMG_Load(name.c_str()));

   if (!this->images[currentI])
   {
      fprintf(stderr, "Error: '%s' could not be opened: %s\n", name.c_str(), IMG_GetError());
      return false;
   }
   else
   {
      if(SDL_SetColorKey(this->images[currentI], SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB(this->images[currentI]->format,255,0,255)) == -1)
      {
         fprintf(stderr, "Warning: colourkey will not be used, reason: %s\n", SDL_GetError());
         return false;
      }
   }
	isConverted.push_back(false);
   return true;
}

void Image::renderImage(uint i,SDL_Surface *dstimg, int destx, int desty)
{
   // Don't render if invisible or if there is no image!
   if(!images.size() || !images[i] || alpha == 0)
   {
      return;
   }

   // Set up blitting area
   SDL_Rect src, dst;
   src.x = images[i]->clip_rect.x;
   src.y = images[i]->clip_rect.y;
   src.w = images[i]->w;
   src.h = images[i]->h;
   dst.x = destx;
   dst.y = desty;
   dst.w = src.w;
   dst.h = src.h;

   // Check if we need to alpha blend
   if(alpha != 255)
   {
      SDL_SetAlpha(images[i], SDL_SRCALPHA, alpha);
   }
	if(!useRotation && !useHardware)
	{
		// Blit the image onto the display surface
		SDL_BlitSurface(images[i], &src, dstimg, &dst);
	}
	else if(useRotation)
	{
		SDL_Surface* tempImage = NULL;
		if(angle)
		{
			tempImage = rotozoomSurface(images[i], angle, scaleX, SMOOTHING_OFF);
		}
		else if(scaleX || scaleY)
		{
			tempImage = zoomSurface(images[i],scaleX, scaleY, SMOOTHING_OFF);
		}
		// Blit the image onto the display surface
		SDL_BlitSurface(tempImage, &src, dstimg, &dst);
		SDL_FreeSurface(tempImage);
	}
	else	//	The only other option is to use HW surfaces
	{
		//	Check if we have converted the current image to a HW Surface already
		if(!isConverted[i])
		{
			//	convert current image
			images[i] = SDL_ConvertSurface(images[i], dstimg->format, SDL_HWSURFACE);
			isConverted[i] = true;
		}
		// Now we blit
		SDL_BlitSurface(images[i], &src, dstimg, &dst);
	}
}

void Image::renderImage(SDL_Surface* dstimg, int destX, int destY)
{
   // Just call the default image
   renderImage(0,dstimg,destX,destY);
}

void Image::screenLock(SDL_Surface* screen)
{
   // Check and lock the surface if necessary.
   if (SDL_MUSTLOCK(screen))
		if (SDL_LockSurface(screen) < 0 )
			return;
}

void Image::setPixel(SDL_Surface *dst, int x, int y, uint r, uint g, uint b,uint a)
{
    Uint32 colour = SDL_MapRGB(dst->format,r,g,b);
    //determine position
    char* pPosition = ( char* ) dst->pixels ;

    //offset by y
    pPosition += ( dst->pitch * y ) ;

    //offset by x
    pPosition += ( dst->format->BytesPerPixel * x ) ;

    //copy pixel data
    memcpy ( pPosition , &colour , dst->format->BytesPerPixel ) ;
}

void Image::setPixel(SDL_Surface *dst, int x, int y, uint r, uint g, uint b)
{
   setPixel(dst,x,y,r,g,b,255);
}

void Image::setPixel(SDL_Surface* dst,int x, int y,Colour c)
{
   setPixel(dst,x,y,c.red,c.green,c.blue,c.alpha);
}

Colour Image::getPixel(SDL_Surface* screen,int x, int y)
{
    //  FIXME
    return Colour(BLACK);
}

void Image::screenUnlock(SDL_Surface* screen)
{
   // Check and unlock the durface if necessary
   if ( SDL_MUSTLOCK(screen) )
		SDL_UnlockSurface(screen);
}

uint Image::size()
{
   return (uint)images.size();
}

void Image::clear()
{
	uint iMax = (uint)images.size();
		for(uint i = 0; i < iMax; i++)
			SDL_FreeSurface(images[i]);
   images.clear();
	isConverted.clear();
}

void Image::setAlpha(uint alpha)
{
	this->alpha = alpha;
}

uint Image::getAlpha()
{
   return alpha;
}

uint Image::getHeight()
{
   return images[0]->h;
}

uint Image::getWidth()
{
   return images[0]->w;
}

void Image::setY(int y)
{
	this->y = y;
}

void Image::setX(int x)
{
	this->x = x;
}

int Image::getX()
{
   return x;
}

int Image::getY()
{
   return y;
}
